/*
 Display ADC on 4 Digit LED Display
 Version: 1.0
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 18/01/2015
 Last Modified: 22/01/2015
 
 Read the ADC and display it on a 4 digit LED display
 
 */

// ATtiny25/45/85 Pin map
//
//             +-\/-+
// Reset PB5  1|o   |8  VCC
// Latch PB3  2|    |7  PB2 SCK
//  ADC2 PB4  3|    |6  PB1 DO
//       GND  4|    |5  PB0
//             +----+

#define F_CPU 1000000 // 1 MHz

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/sleep.h>
#include "setup.c"

#define adcPin 2

volatile uint8_t numberDisplay[4] = {0, 0, 0, 0}; // Storage of the number to display
volatile uint8_t segmentUpto = 0; // Which segment to light up

// Shift out the numbers on the segment displays, no need to delay because the timer does that for us
void display_led(uint8_t first, uint8_t second, uint8_t third, uint8_t fourth) {
	if (segmentUpto == 0) {
		latch_low;
		spi_transfer(0x0E); // First segment GND
		spi_transfer(number_to_led[first]);
		latch_high;
	}
	else if (segmentUpto == 1) {
		latch_low;
		spi_transfer(0x0D);
		spi_transfer(number_to_led[second] | (1<<NDP)); // Add dot poin
		latch_high;
	}
	else if (segmentUpto == 2) {
		latch_low;
		spi_transfer(0x0B);
		spi_transfer(number_to_led[third]);
		latch_high;
	}
	else if (segmentUpto == 3) {
		latch_low;
		spi_transfer(0x07);
		spi_transfer(number_to_led[fourth]);
		latch_high;
		segmentUpto = 255; // Reset to zero on next increment
	}
	segmentUpto++;
}

// Store the number to be displayed on the screen
void store_number(int number) {
	numberDisplay[0] = BLANK; // First always blank
	
	if ((number / 100) >= 1) {
		numberDisplay[1] = number / 100;
	}
	else {
		numberDisplay[1] = BLANK;
	}
	
	if (((number % 100) / 10) >= 1) {
		numberDisplay[2] = (number % 100) / 10;
	}
	else if (numberDisplay[1] == BLANK) { // 2nd digit blank?
		numberDisplay[2] = BLANK;
	}
	else { // 2nd digit isn't blank, then display a 0
		numberDisplay[2] = 0;
	}
	
	numberDisplay[3] = number % 10;
}

int main(void) {
	setup();
	
	while(1) {
		// Read voltage
		int adcValue = analog_read_average(adcPin);
		double calculateDouble = (double) ((double) (adcValue) * 0.97); // Calibrate this for higher accuracy
		store_number((int) calculateDouble);
		
		_delay_ms(100);
	}
}

EMPTY_INTERRUPT(ADC_vect);

// Timer0 overflow after 2ms, used to update LED display
ISR(TIM0_OVF_vect) {
	display_led(numberDisplay[0], numberDisplay[1], numberDisplay[2], numberDisplay[3]);
}
