/*
 Small Programmable Power Supply
 Version: 1.0
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 8/09/2013
 Last Modified: 13/05/2014

 */
 
#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif

#ifndef boolean
	typedef uint8_t boolean;
#endif
#ifndef bool
	typedef uint8_t bool;
#endif 
#ifndef byte
	typedef uint8_t byte;
#endif

#ifndef NULL
#define NULL ((void *)0)
#endif

#define LOW 0
#define HIGH 1
#define false 0
#define true 1

#define latch PB3
#define latch_high	PORTB |=  (1<<latch);
#define latch_low		PORTB &= ~(1<<latch);

// LED A, B, C pin map to 595 Q0-7 pins
#define NA 7
#define NB 0
#define NC 2
#define ND 4
#define NE 5
#define NF 6
#define NG 1
#define NDP 3

#define BLANK 10

// Mapping of characters to LEDs 
uint8_t number_to_led[19] = {
	(1<<NA) | (1<<NF) | (1<<NE) | (1<<ND) | (1<<NC) | (1<<NB), // 0
	(1<<NB) | (1<<NC), // 1
	(1<<NA) | (1<<NB) | (1<<NG) | (1<<NE) | (1<<ND), // 2
	(1<<NA) | (1<<NB) | (1<<NG) | (1<<NC) | (1<<ND), // 3
	(1<<NF) | (1<<NG) | (1<<NB) | (1<<NC), // 4
	(1<<NA) | (1<<NF) | (1<<NG) | (1<<NC) | (1<<ND), // 5
	(1<<NA) | (1<<NF) | (1<<NG) | (1<<NE) | (1<<ND) | (1<<NC), // 6
	(1<<NA) | (1<<NB) | (1<<NC), // 7
	(1<<NA) | (1<<NB) | (1<<NC) | (1<<ND) | (1<<NE) | (1<<NF) | (1<<NG), // 8
	(1<<NA) | (1<<NF) | (1<<NG) | (1<<NB) | (1<<NC), // 9
	0, // 10 (Blank)
};

// SPI transfer 1 byte and return the result
uint8_t spi_transfer(uint8_t data) {
	USIDR = data;
	USISR = _BV(USIOIF); // clear flag

	while ( (USISR & _BV(USIOIF)) == 0 ) {
		USICR = (1<<USIWM0)|(1<<USICS1)|(1<<USICLK)|(1<<USITC);
	}
	return USIDR;
}

// ADC Read
int analog_read(uint8_t pin) {
	uint8_t low, high;
	
	ADMUX = (1 << 7) | (pin & 0x3f); // 1.1V voltage reference
	
	set_sleep_mode(SLEEP_MODE_ADC); // Set sleep mode ADC
	sleep_mode(); // System sleeps here
	
	// ADCL must be read first, then ADCH
	low = ADCL;
	high = ADCH;
	
	// combine the two bytes
	return (high << 8) | low;
}

// ADC Read with averaging of 5 readings
int analog_read_average(uint8_t pin) {
	int adcreading = 0;
	for (uint8_t x = 0; x < 5; x++) {
		adcreading = adcreading + analog_read(pin);
	}
	adcreading = adcreading / 5;
	
	return adcreading;
}

// Setup the ATtiny
void setup(void) {
	// Initialise the SPI (without DI)
	DDRB |= (1<<PB2); // SPI CLK
	DDRB |= (1<<PB1); // SPI DO
	DDRB |= (1<<latch); // Latch
	
	// Set ADC prescale factor to 8
	// 1 MHz / 8 = 125 KHz, inside the desired 50-200 KHz range.
	sbi(ADCSRA, ADPS1);
	sbi(ADCSRA, ADPS0);
	
	// Turn on ADC interrupt
	sbi(ADCSRA, ADIE);
	
	// Turn on ADC
	sbi(ADCSRA, ADEN);
	
	// Turn on Timer0 overflow interrupt
	sbi(TIMSK, TOIE0);
	
	// Set Timer0 to 8 prescaler
	TCCR0B = (1<<CS01);
	
	// Turn on interrupts
	sei();
}
